#include "Qcm/query/query.hpp"
#include "Qcm/status/provider_status.hpp"
#include "Qcm/app.hpp"

namespace qcm
{

Query::Query(QObject* parent): QAsyncResult(parent), m_delay(true) {
    setForwardError(true);
    m_timer.setSingleShot(true);
    connect(&m_timer, &QTimer::timeout, this, &Query::reload);
}
Query::~Query() {}

auto Query::create(std::string_view name) -> Query* {
    auto q = QMetaType::fromName(name).create();
    _assert_rel_(q);
    return static_cast<Query*>(q);
}

void Query::request_reload() {
    if (delay()) {
        m_timer.setInterval(100);
        m_timer.start();
    } else {
        reload();
    }
}

auto Query::delay() const -> bool { return m_delay; }
void Query::setDelay(bool v) {
    if (ycore::cmp_exchange(m_delay, v)) {
        delayChanged();
    }
}
void Query::connectSyncFinished() {
    connect(App::instance()->provider_status(),
            &ProviderStatusModel::syncingChanged,
            this,
            [this](bool syncing) {
                if (syncing) return;
                request_reload();
            });
}

QueryList::QueryList(QObject* parent)
    : Query(parent), m_offset(0), m_limit(200), m_sort(0), m_asc(true) {
    // connect_requet_reload(&QueryList::offsetChanged);
    // connect_requet_reload(&QueryList::limitChanged);
}
QueryList::~QueryList() {}
auto QueryList::offset() const -> qint32 { return m_offset; }
auto QueryList::limit() const -> qint32 { return m_limit; }

void QueryList::setOffset(qint32 v) {
    if (ycore::cmp_exchange(m_offset, v)) {
        offsetChanged();
    }
}
void QueryList::setLimit(qint32 v) {
    if (ycore::cmp_exchange(m_offset, v)) {
        limitChanged();
    }
}
auto QueryList::sort() const -> qint32 { return m_sort; }
void QueryList::setSort(qint32 v) {
    if (ycore::cmp_exchange(m_sort, v)) {
        sortChanged();
    }
}
auto QueryList::asc() const -> bool { return m_asc; }
void QueryList::setAsc(bool v) {
    if (ycore::cmp_exchange(m_asc, v)) {
        ascChanged();
    }
}

void QueryList::fetchMore(qint32) { log::warn("fetchMore not impl"); }

void detail::try_connect_fetch_more(QObject* query, QObject* model) {
    if (model == nullptr) return;

    auto signal    = QMetaObject::normalizedSignature("reqFetchMore(qint32)");
    auto slot      = QMetaObject::normalizedSignature("fetchMore(qint32)");
    auto signalIdx = model->metaObject()->indexOfSignal(signal);
    auto slotIdx   = query->metaObject()->indexOfSlot(slot);
    if (signalIdx != -1 && slotIdx != -1) {
        QObject::connect(model, SIGNAL(reqFetchMore(qint32)), query, SLOT(fetchMore(qint32)));
    } else if (signalIdx != -1 || slotIdx != -1) {
        log::warn("reqFetchMore not connected");
    }
}

} // namespace qcm

#include <Qcm/query/moc_query.cpp>